-- mlys.lua controller user script
-- an example for IRCAM Forum 2023

require("tools")

cos=math.cos
sin=math.sin
π=math.pi
R1 = 30
R2 = 10

-- This function is called from within the 'initialize' function
function createMesh()
-- Here is our object's geometry, defined as a parametric surface. NB: in the surface definitition, (s,t) ∈ [0...1]x[0...1]
-- This a cylinder which base is an ellipse.
	-- a few parameters:
	local ɑ = 1.2
	local β = 0.8
	local γ = 1
	-- equation:
	local surface=ParametricSurface3D:new{ scale=0.5, resolution_s=R1, resolution_t=R2,
									 f=function(s,t)
											return ɑ*cos(2*π*s), β*sin(2*π*s), γ*t
										end }
    -- M is a global variable!
	M = surface:createMesh()
	--view(M)
end

-- This function is called from within the 'initialize' function
function createSoundingObject(name)
	
	local points = {R1*R2/2,1+R1*R2/2}
	local holdingMesh = modalys.create_mesh{ kind="restrict-points", mesh=M, points=points }
	
    -- obj is a global variable!
	obj = modalys.create_object{ name=name,kind="finite-element",mesh=M,block=holdingMesh,modes=60,
							     	   thickness=0.02,material="steel",constloss=2.3,freqloss=2.2}
end

-- This function is called from within the 'initialize' function
function addOutputPoints()
	-- Add output (listening) points
	local accOut1 = create_access{where=obj,name="accessOut1",location=R1*R2*0.23}
	local accOut2 = create_access{where=obj,name="accessOut2",location=R1*R2*0.48}
	modalys.create_point_output{where=accOut1,name="output1",channel=2}
	modalys.create_point_output{where=accOut2,name="output2",channel=1}
end

-- This function is called from within the 'initialize' function
function addSignalInput()
	-- Add one or several signal input to excite the object
	local signalToForce = modalys.create_input_force_signal_access{where=obj,position=R1*R2*0.77,kind="normal",name="mySignalToForce",channel=1}
end

-- This function is called from a 'luafunction' message:
function changeInputLocation()
	local nn = get_info("num-nodes",obj)
	local v = math.random(nn)
	set_value{ctrl="mySignalToForce@position", value=v}
	view(obj)
end

-- This function is called once upon bang:
function initialize()
	createMesh()
	local name = "myObject"
	createSoundingObject(name)
	addOutputPoints()
	addSignalInput()
	print("initialized!")
end

function update()
	-- do update stuff
end